"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LwrServerUtils = void 0;
const CommonUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/CommonUtils");
const lwr_1 = require("lwr");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const os_1 = __importDefault(require("os"));
const defaultServerTimeout = 30 * 60 * 1000;
class LwrServerUtils {
    static async startLwrServer(componentName, projectDir, idleTimeout = defaultServerTimeout, exitOnIdle = true) {
        const lwrApp = lwr_1.createServer(LwrServerUtils.getMergedLwrConfig(componentName, projectDir));
        const runtimeConfig = lwrApp.getConfig();
        return lwrApp
            .listen(() => {
            console.log(`Listening on port ${runtimeConfig.port} ( mode = ${runtimeConfig.serverMode} , type = ${runtimeConfig.serverType} )`);
            LwrServerUtils.setServerIdleTimeout(lwrApp, idleTimeout, () => {
                if (exitOnIdle) {
                    process.exit(0);
                }
            });
        })
            .then(() => {
            return Promise.resolve(`${runtimeConfig.port}`);
        });
    }
    static getMergedLwrConfig(componentName, projectDir) {
        const nextPort = LwrServerUtils.getNextServerPort();
        const rootDirectory = path_1.default.resolve(projectDir);
        const cacheDirectory = path_1.default.resolve(`${projectDir}/__lwr_cache__`);
        const modifiedModuleProviders = LwrServerUtils.getModifiedModuleProviders();
        const componentFullPath = path_1.default.resolve(path_1.default.join(projectDir, componentName));
        const twoLevelUp = path_1.default.resolve(path_1.default.join(componentFullPath, '../../'));
        const rootComp = componentFullPath
            .replace(`${twoLevelUp}${path_1.default.sep}`, '')
            .replace(/\\/gi, '/');
        const lwcModuleRecord = {
            dir: twoLevelUp
        };
        const defaultLwrRoute = {
            id: rootComp,
            path: '/',
            rootComponent: rootComp
        };
        let config = {};
        try {
            config = CommonUtils_1.CommonUtils.loadJsonFromFile(path_1.default.resolve(path_1.default.join(projectDir, 'lwr.config.json')));
        }
        catch {
        }
        if (!config.port && nextPort) {
            config.port = nextPort;
        }
        if (!config.rootDir) {
            config.rootDir = rootDirectory;
        }
        if (!config.cacheDir) {
            config.cacheDir = cacheDirectory;
        }
        if (!config.lwc) {
            config.lwc = {
                modules: [lwcModuleRecord]
            };
        }
        else {
            config.lwc.modules.unshift(lwcModuleRecord);
        }
        if (!config.moduleProviders) {
            config.moduleProviders = modifiedModuleProviders;
        }
        else {
            config.moduleProviders.unshift(...modifiedModuleProviders);
        }
        if (!config.routes) {
            config.routes = [defaultLwrRoute];
        }
        else {
            config.routes.unshift(defaultLwrRoute);
        }
        return config;
    }
    static setServerIdleTimeout(app, timeout, callback) {
        const server = app['server'];
        if (server) {
            let timer = LwrServerUtils.setTimeoutTimer(app, timeout, callback);
            server.on('request', () => {
                if (timer) {
                    clearTimeout(timer);
                }
                timer = LwrServerUtils.setTimeoutTimer(app, timeout, callback);
            });
        }
    }
    static getNextServerPort() {
        try {
            const getUsedTCPPortsCommand = process.platform === 'win32'
                ? 'netstat -ano -p tcp | findstr "LISTENING"'
                : 'lsof -Pn -iTCP -sTCP:LISTEN | grep TCP';
            const results = CommonUtils_1.CommonUtils.executeCommandSync(getUsedTCPPortsCommand).split('\n');
            const ports = [];
            for (const result of results) {
                try {
                    let idxStart = result.indexOf('TCP') + 3;
                    idxStart = result.indexOf(':', idxStart) + 1;
                    const idxEnd = result.indexOf(' ', idxStart);
                    const portString = result.substring(idxStart, idxEnd);
                    const portNumber = parseInt(portString, 10);
                    if (!Number.isNaN(portNumber)) {
                        ports.push(portNumber);
                    }
                }
                catch {
                }
            }
            ports.sort((a, b) => (a > b ? 1 : -1));
            const largest = ports[ports.length - 1];
            if (largest) {
                return largest + 2;
            }
            else {
                return undefined;
            }
        }
        catch {
            return undefined;
        }
    }
    static getModifiedModuleProviders() {
        const cacheDirectory = path_1.default.join(os_1.default.tmpdir(), '__temporary_cache_to_be_deleted__');
        const tempServer = lwr_1.createServer({
            cacheDir: cacheDirectory
        });
        const config = tempServer.getConfig();
        tempServer.close();
        fs_1.default.rmdirSync(cacheDirectory, { recursive: true });
        const newProviders = config.moduleProviders.map((provider) => provider[0] === '@lwrjs/lwc-module-provider'
            ? [
                path_1.default.resolve(`${__dirname}/InternalLwcModuleProvider.js`),
                undefined
            ]
            : provider);
        newProviders.push([
            path_1.default.resolve(`${__dirname}/CustomLwcModuleProvider.js`),
            undefined
        ]);
        return newProviders;
    }
    static setTimeoutTimer(lwrApp, timeoutMilliseconds, callback) {
        return setTimeout(async () => {
            const timeoutSeconds = timeoutMilliseconds / 1000;
            console.log(`Server idle for ${timeoutSeconds} seconds... shutting down`);
            await lwrApp.close();
            callback();
        }, timeoutMilliseconds);
    }
}
exports.LwrServerUtils = LwrServerUtils;
//# sourceMappingURL=LwrServerUtils.js.map